/*----------------------------------------------------------------------------
 *      RL-ARM - RTX
 *----------------------------------------------------------------------------
 *      Name:    BLINKY.C
 *      Purpose: RTX example program
 *----------------------------------------------------------------------------
 *      This code is part of the RealView Run-Time Library.
 *      Copyright (c) 2006-2012 KEIL - An ARM Company. All rights reserved.
 *---------------------------------------------------------------------------*/
#include "cmsis_os.h"
#include "MK64F12.h"

osThreadId tid_phaseA;                  /* Thread id of thread: phase_a      */
osThreadId tid_phaseB;                  /* Thread id of thread: phase_b      */
osThreadId tid_phaseC;                  /* Thread id of thread: phase_c      */
osThreadId tid_phaseD;                  /* Thread id of thread: phase_c      */
osThreadId tid_clock;                   /* Thread id of thread: clock        */

#define LED_NUM     3                   /* Number of user LEDs                */

const uint32_t led_mask[] = { 1UL << 22,
                              1UL << 21,
                              1UL << 26 };

GPIO_Type * led_addr[]    = { PTB,
                              PTB,
                              PTE };

#define LED_RED    0
#define LED_BLUE   1
#define LED_GREEN  2

/*----------------------------------------------------------------------------
  Function that initializes LEDs
 *----------------------------------------------------------------------------*/
void LED_Initialize(void) {

  SIM->SCGC5     |= ((1UL <<  13) | (1UL <<  10));   /* Enable Port B,E Clock */
  PORTB->PCR[22]  = (1UL <<  8);                     /* PTB22 is GPIO pin     */
  PORTB->PCR[21]  = (1UL <<  8);                     /* PTB21 is GPIO pin     */
  PORTE->PCR[26]  = (1UL <<  8);                     /* PTE26 is GPIO pin     */

  /* Switch LEDs off and enable output                                        */
  PTB->PDOR = (led_mask[1] | led_mask[0]);
  PTE->PDOR = led_mask[2];
  PTB->PDDR = (led_mask[1] | led_mask[0]);
  PTE->PDDR = led_mask[2];
}

/*----------------------------------------------------------------------------
  Function that turns on Red LED
 *----------------------------------------------------------------------------*/
void LEDRed_On (void) {
  led_addr[LED_BLUE]->PSOR  = led_mask[LED_BLUE];   /* Blue LED Off */
  led_addr[LED_GREEN]->PSOR = led_mask[LED_GREEN];  /* Green LED Off*/
  led_addr[LED_RED]->PCOR   = led_mask[LED_RED];    /* Red LED On   */
}

/*----------------------------------------------------------------------------
  Function that turns on Green LED
 *----------------------------------------------------------------------------*/
void LEDGreen_On (void) {
  led_addr[LED_RED]->PSOR   = led_mask[LED_RED];   /* Red LED Off   */
  led_addr[LED_BLUE]->PSOR  = led_mask[LED_BLUE];  /* Blue LED Off  */
  led_addr[LED_GREEN]->PCOR = led_mask[LED_GREEN]; /* Green LED On  */
}

/*----------------------------------------------------------------------------
  Function that turns on Blue LED
 *----------------------------------------------------------------------------*/
void LEDBlue_On (void) {
  led_addr[LED_RED]->PSOR   = led_mask[LED_RED];   /* Red LED Off   */
  led_addr[LED_GREEN]->PSOR = led_mask[LED_GREEN]; /* Green LED Off */
  led_addr[LED_BLUE]->PCOR  = led_mask[LED_BLUE];  /* Blue LED On   */
}

/*----------------------------------------------------------------------------
 *      Function 'signal_func' called from multiple threads
 *---------------------------------------------------------------------------*/
void signal_func (osThreadId tid)  {
  osSignalSet(tid_clock, 0x0100);           /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osSignalSet(tid_clock, 0x0100);           /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osSignalSet(tid, 0x0001);                 /* set signal to thread 'thread' */
  osDelay(500);                             /* delay 500ms                   */
}

/*----------------------------------------------------------------------------
 *      Thread 1 'phaseA': Phase A output
 *---------------------------------------------------------------------------*/
void phaseA (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LEDRed_On();
    osDelay(500);                           /* delay 500ms                   */
    signal_func(tid_phaseB);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 *      Thread 2 'phaseB': Phase B output
 *---------------------------------------------------------------------------*/
void phaseB (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LEDBlue_On();
    osDelay(500);                           /* delay 500ms                   */
    signal_func(tid_phaseC);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 *      Thread 3 'phaseC': Phase C output
 *---------------------------------------------------------------------------*/
void phaseC (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LEDRed_On();
    osDelay(500);                           /* delay 500ms                   */
    signal_func(tid_phaseD);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 *      Thread 4 'phaseD': Phase D output
 *---------------------------------------------------------------------------*/
void phaseD (void  const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    LEDBlue_On();
    osDelay(500);                           /* delay 500ms                   */
    signal_func(tid_phaseA);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 *      Thread 5 'clock': Signal Clock
 *---------------------------------------------------------------------------*/
void clock (void  const *argument) {
  for (;;) {
    osSignalWait(0x0100, osWaitForever);    /* wait for an event flag 0x0100 */
    LEDGreen_On();
    osDelay(200);                           /* delay 200ms                   */
  }
}



osThreadDef(phaseA, osPriorityNormal, 1, 0);
osThreadDef(phaseB, osPriorityNormal, 1, 0);
osThreadDef(phaseC, osPriorityNormal, 1, 0);
osThreadDef(phaseD, osPriorityNormal, 1, 0);
osThreadDef(clock,  osPriorityNormal, 1, 0);

/*----------------------------------------------------------------------------
 *      Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
int main (void) {

  LED_Initialize();                         /* Initialize the LEDs           */
  tid_phaseA = osThreadCreate(osThread(phaseA), NULL);
  tid_phaseB = osThreadCreate(osThread(phaseB), NULL);
  tid_phaseC = osThreadCreate(osThread(phaseC), NULL);
  tid_phaseD = osThreadCreate(osThread(phaseD), NULL);
  tid_clock  = osThreadCreate(osThread(clock),  NULL);

  osSignalSet(tid_phaseA, 0x0001);          /* set signal to phaseA thread   */

  osDelay(osWaitForever);
  while(1);
}
