/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher for MCB1700
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2004-2014 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include "MK64F12.h"

uint32_t LEDOn, LEDOff; 

#define NUM_LEDS  4                     /* Number of LEDs                     */

const uint32_t led_mask[] = { 1UL << 6,
                              1UL << 7,
                              1UL << 8,
                              1UL << 9 };

#define LED_A      0
#define LED_B      1
#define LED_C      2
#define LED_D      0
#define LED_CLK    3

/*----------------------------------------------------------------------------
  Function that initializes LEDs
 *----------------------------------------------------------------------------*/
void LED_Initialize(void) {
  SIM->SCGC5    |= (1UL <<  13);       /* Enable Port E Clock                */
  PORTE->PCR[6] = (1UL <<  8);         /* PTE6 is GPIO pin                  */
  PORTE->PCR[7] = (1UL <<  8);         /* PTE7 is GPIO pin                  */
  PORTE->PCR[8] = (1UL <<  8);         /* PTE8 is GPIO pin                  */
  PORTE->PCR[9] = (1UL <<  8);         /* PTE9 is GPIO pin                  */

  /* Switch LEDs off and enable output                                        */
  PTE->PDOR = (led_mask[3] | led_mask[2] | led_mask[1] | led_mask[0]);
  PTE->PDDR = (led_mask[3] | led_mask[2] | led_mask[1] | led_mask[0]);
}

/*----------------------------------------------------------------------------
  Function that turns on requested LED
 *----------------------------------------------------------------------------*/
void LED_On (unsigned int idx) {
  if (idx < NUM_LEDS) {
    PTE->PCOR = led_mask[idx];
  }
}

/*----------------------------------------------------------------------------
  Function that turns off requested LED
 *----------------------------------------------------------------------------*/
void LED_Off (unsigned int idx) {
  if (idx < NUM_LEDS) {
    PTE->PSOR = led_mask[idx];
  }
}

/*----------------------------------------------------------------------------
  SysTick IRQ Handler
 *----------------------------------------------------------------------------*/
void SysTick_Handler (void) {
  static uint32_t ticks;
  
  switch (ticks++) {
    case   0: LEDOn  = 1; break;
    case  50: LEDOff = 1; break;
    case  90: ticks  = 0; break;
    
    default:
      if (ticks > 100) {
        ticks = 0;
      }
  }
}

/*----------------------------------------------------------------------------
  Main function
 *----------------------------------------------------------------------------*/
int main (void) {
  int32_t max_num = NUM_LEDS - 1;
  int32_t num = 0;
  int32_t dir = 1;

  SystemCoreClockUpdate ();                 /* Update system core clock       */  
  SysTick_Config(SystemCoreClock/100);      /* Generate interrupt each 10 ms  */
  LED_Initialize();                         /* LED Initialization             */

  while (1) {
    if (LEDOn) {
      LEDOn = 0;
      LED_On (num);                         /* Turn specified LED on          */
    }

    if (LEDOff) {
      LEDOff = 0;
      LED_Off (num);                        /* Turn specified LED off         */

      num += dir;                           /* Change LED number              */
      
      if (dir == 1 && num == max_num) {
        dir = -1;                           /* Change direction to down       */
      }
      else if (num == 0) {
        dir =  1;                           /* Change direction to up         */
      }
    }
  }
}
